@@ -18,6 +18,7 @@
  */
 package org.apache.parquet.cli.commands;
 
+import java.io.File;
 import org.apache.commons.logging.LogFactory;
 import org.apache.log4j.PropertyConfigurator;
 import org.junit.Rule;
@@ -25,8 +26,6 @@ import org.junit.rules.TemporaryFolder;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.io.File;
-
 public abstract class FileTest {
 
   static final String INT32_FIELD = "int32_field";
@@ -47,12 +46,10 @@ public abstract class FileTest {
   }
 
   protected static Logger createLogger() {
-    PropertyConfigurator.configure(
-      ParquetFileTest.class.getResource("/cli-logging.properties"));
+    PropertyConfigurator.configure(ParquetFileTest.class.getResource("/cli-logging.properties"));
     Logger console = LoggerFactory.getLogger(ParquetFileTest.class);
-    LogFactory.getFactory().setAttribute(
-      "org.apache.commons.logging.Log",
-      "org.apache.commons.logging.impl.Log4JLogger");
+    LogFactory.getFactory()
+        .setAttribute("org.apache.commons.logging.Log", "org.apache.commons.logging.impl.Log4JLogger");
     return console;
   }
 }
