@@ -6,9 +6,9 @@
  * to you under the Apache License, Version 2.0 (the
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
- *
- *   http://www.apache.org/licenses/LICENSE-2.0
- *
+ * <p>
+ * http://www.apache.org/licenses/LICENSE-2.0
+ * <p>
  * Unless required by applicable law or agreed to in writing,
  * software distributed under the License is distributed on an
  * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
@@ -35,8 +35,7 @@ public class TestPreconditions {
       Preconditions.checkArgument(false, "Test message");
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalArgumentException e) {
-      Assert.assertEquals("Should format message",
-          "Test message", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message", e.getMessage());
     }
   }
 
@@ -52,8 +51,7 @@ public class TestPreconditions {
       Preconditions.checkArgument(false, "Test message %s", 12);
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalArgumentException e) {
-      Assert.assertEquals("Should format message",
-          "Test message 12", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message 12", e.getMessage());
     }
   }
 
@@ -69,8 +67,7 @@ public class TestPreconditions {
       Preconditions.checkArgument(false, "Test message %s %s", 12, null);
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalArgumentException e) {
-      Assert.assertEquals("Should format message",
-          "Test message 12 null", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message 12 null", e.getMessage());
     }
   }
 
@@ -86,8 +83,7 @@ public class TestPreconditions {
       Preconditions.checkArgument(false, "Test message %s %s %s", 12, null, "column");
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalArgumentException e) {
-      Assert.assertEquals("Should format message",
-          "Test message 12 null column", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message 12 null column", e.getMessage());
     }
   }
 
@@ -103,8 +99,7 @@ public class TestPreconditions {
       Preconditions.checkArgument(false, "Test message %s %s %s %s", 12, null, "column", true);
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalArgumentException e) {
-      Assert.assertEquals("Should format message",
-          "Test message 12 null column true", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message 12 null column true", e.getMessage());
     }
   }
 
@@ -120,10 +115,9 @@ public class TestPreconditions {
       Preconditions.checkArgument(false, "Test message %d", 12);
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalArgumentException e) {
-      Assert.assertEquals("%d is not a valid format option",
-          "d != java.lang.String", e.getMessage());
+      Assert.assertEquals("%d is not a valid format option", "d != java.lang.String", e.getMessage());
     }
-  }  
+  }
 
   @Test
   public void testCheckState() {
@@ -137,8 +131,7 @@ public class TestPreconditions {
       Preconditions.checkState(false, "Test message: %s %s", 12, null);
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalStateException e) {
-      Assert.assertEquals("Should format message",
-          "Test message: 12 null", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message: 12 null", e.getMessage());
     }
   }
 
@@ -154,8 +147,7 @@ public class TestPreconditions {
       Preconditions.checkState(false, "Test message");
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalStateException e) {
-      Assert.assertEquals("Should format message",
-          "Test message", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message", e.getMessage());
     }
   }
 
@@ -171,8 +163,7 @@ public class TestPreconditions {
       Preconditions.checkState(false, "Test message %s", 12);
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalStateException e) {
-      Assert.assertEquals("Should format message",
-          "Test message 12", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message 12", e.getMessage());
     }
   }
 
@@ -188,8 +179,7 @@ public class TestPreconditions {
       Preconditions.checkState(false, "Test message %s %s", 12, null);
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalStateException e) {
-      Assert.assertEquals("Should format message",
-          "Test message 12 null", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message 12 null", e.getMessage());
     }
   }
 
@@ -205,8 +195,7 @@ public class TestPreconditions {
       Preconditions.checkState(false, "Test message %s %s %s", 12, null, "column");
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalStateException e) {
-      Assert.assertEquals("Should format message",
-          "Test message 12 null column", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message 12 null column", e.getMessage());
     }
   }
 
@@ -222,10 +211,9 @@ public class TestPreconditions {
       Preconditions.checkState(false, "Test message %s %s %s %s", 12, null, "column", true);
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalStateException e) {
-      Assert.assertEquals("Should format message",
-          "Test message 12 null column true", e.getMessage());
+      Assert.assertEquals("Should format message", "Test message 12 null column true", e.getMessage());
     }
-  }  
+  }
 
   @Test
   public void checkStateMessageOnlySupportsStringTypeTemplate() {
@@ -239,8 +227,7 @@ public class TestPreconditions {
       Preconditions.checkState(false, "Test message %d", 12);
       Assert.fail("Should throw exception when isValid is false");
     } catch (IllegalArgumentException e) {
-      Assert.assertEquals("%d is not a valid format option",
-          "d != java.lang.String", e.getMessage());
+      Assert.assertEquals("%d is not a valid format option", "d != java.lang.String", e.getMessage());
     }
-  }  
+  }
 }
