@@ -34,8 +34,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
-
-import org.apache.hadoop.conf.Configurable;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.BlockLocation;
 import org.apache.hadoop.fs.FileStatus;
@@ -49,9 +47,10 @@ import org.apache.hadoop.mapreduce.JobContext;
 import org.apache.hadoop.mapreduce.RecordReader;
 import org.apache.hadoop.mapreduce.TaskAttemptContext;
 import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
-
 import org.apache.hadoop.mapreduce.lib.input.FileSplit;
 import org.apache.parquet.Preconditions;
+import org.apache.parquet.conf.HadoopParquetConfiguration;
+import org.apache.parquet.conf.ParquetConfiguration;
 import org.apache.parquet.filter.UnboundRecordFilter;
 import org.apache.parquet.filter2.compat.FilterCompat;
 import org.apache.parquet.filter2.compat.FilterCompat.Filter;
@@ -76,19 +75,18 @@ import org.slf4j.LoggerFactory;
 
 /**
  * The input format to read a Parquet file.
- *
+ * <p>
  * It requires an implementation of {@link ReadSupport} to materialize the records.
- *
+ * <p>
  * The requestedSchema will control how the original records get projected by the loader.
  * It must be a subset of the original schema. Only the columns needed to reconstruct the records with the requestedSchema will be scanned.
  *
+ * @param <T> the type of the materialized records
  * @see #READ_SUPPORT_CLASS
  * @see #UNBOUND_RECORD_FILTER
  * @see #STRICT_TYPE_CHECKING
  * @see #FILTER_PREDICATE
  * @see #TASK_SIDE_METADATA
- *
- * @param <T> the type of the materialized records
  */
 public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
 
@@ -144,6 +142,11 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
    */
   public static final String BLOOM_FILTERING_ENABLED = "parquet.filter.bloom.enabled";
 
+  /**
+   * Key to configure if off-heap buffer should be used for decryption
+   */
+  public static final String OFF_HEAP_DECRYPT_BUFFER_ENABLED = "parquet.decrypt.off-heap.buffer.enabled";
+
   /**
    * key to turn on or off task side metadata loading (default true)
    * if true then metadata is read on the task side and some tasks may finish immediately.
@@ -158,7 +161,18 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
 
   private static final int MIN_FOOTER_CACHE_SIZE = 100;
 
-  public static void setTaskSideMetaData(Job job,  boolean taskSideMetadata) {
+  /**
+   * Key to enable/disable vectored io while reading parquet files:
+   * {@value}.
+   */
+  public static final String HADOOP_VECTORED_IO_ENABLED = "parquet.hadoop.vectored.io.enabled";
+
+  /**
+   * Default value of parquet.hadoop.vectored.io.enabled is {@value}.
+   */
+  public static final boolean HADOOP_VECTORED_IO_DEFAULT = false;
+
+  public static void setTaskSideMetaData(Job job, boolean taskSideMetadata) {
     ContextUtil.getConfiguration(job).setBoolean(TASK_SIDE_METADATA, taskSideMetadata);
   }
 
@@ -166,13 +180,14 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
     return configuration.getBoolean(TASK_SIDE_METADATA, TRUE);
   }
 
-  public static void setReadSupportClass(Job job,  Class<?> readSupportClass) {
+  public static void setReadSupportClass(Job job, Class<?> readSupportClass) {
     ContextUtil.getConfiguration(job).set(READ_SUPPORT_CLASS, readSupportClass.getName());
   }
 
   public static void setUnboundRecordFilter(Job job, Class<? extends UnboundRecordFilter> filterClass) {
     Configuration conf = ContextUtil.getConfiguration(job);
-    checkArgument(getFilterPredicate(conf) == null,
+    checkArgument(
+        getFilterPredicate(conf) == null,
         "You cannot provide an UnboundRecordFilter after providing a FilterPredicate");
 
     conf.set(UNBOUND_RECORD_FILTER, filterClass.getName());
@@ -188,21 +203,16 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
     return ConfigurationUtil.getClassFromConfig(configuration, UNBOUND_RECORD_FILTER, UnboundRecordFilter.class);
   }
 
-  private static UnboundRecordFilter getUnboundRecordFilterInstance(Configuration configuration) {
-    Class<?> clazz = ConfigurationUtil.getClassFromConfig(configuration, UNBOUND_RECORD_FILTER, UnboundRecordFilter.class);
-    if (clazz == null) { return null; }
-
+  private static UnboundRecordFilter getUnboundRecordFilterInstance(ParquetConfiguration configuration) {
+    Class<?> clazz =
+        ConfigurationUtil.getClassFromConfig(configuration, UNBOUND_RECORD_FILTER, UnboundRecordFilter.class);
+    if (clazz == null) {
+      return null;
+    }
     try {
-      UnboundRecordFilter unboundRecordFilter = (UnboundRecordFilter) clazz.newInstance();
-
-      if (unboundRecordFilter instanceof Configurable) {
-        ((Configurable)unboundRecordFilter).setConf(configuration);
-      }
-
-      return unboundRecordFilter;
+      return (UnboundRecordFilter) clazz.newInstance();
     } catch (InstantiationException | IllegalAccessException e) {
-      throw new BadConfigurationException(
-          "could not instantiate unbound record filter class", e);
+      throw new BadConfigurationException("could not instantiate unbound record filter class", e);
     }
   }
 
@@ -215,7 +225,8 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
   }
 
   public static void setFilterPredicate(Configuration configuration, FilterPredicate filterPredicate) {
-    checkArgument(getUnboundRecordFilter(configuration) == null,
+    checkArgument(
+        getUnboundRecordFilter(configuration) == null,
         "You cannot provide a FilterPredicate after providing an UnboundRecordFilter");
 
     configuration.set(FILTER_PREDICATE + ".human.readable", filterPredicate.toString());
@@ -227,6 +238,10 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
   }
 
   private static FilterPredicate getFilterPredicate(Configuration configuration) {
+    return getFilterPredicate(new HadoopParquetConfiguration(configuration));
+  }
+
+  private static FilterPredicate getFilterPredicate(ParquetConfiguration configuration) {
     try {
       return SerializationUtil.readObjectFromConfAsBase64(FILTER_PREDICATE, configuration);
     } catch (IOException e) {
@@ -242,6 +257,10 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
    * @return a filter for the unbound record filter specified in conf
    */
   public static Filter getFilter(Configuration conf) {
+    return getFilter(new HadoopParquetConfiguration(conf));
+  }
+
+  public static Filter getFilter(ParquetConfiguration conf) {
     return FilterCompat.get(getFilterPredicate(conf), getUnboundRecordFilterInstance(conf));
   }
 
@@ -264,7 +283,7 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
    * the read support property in their configuration.
    *
    * @param readSupportClass a ReadSupport subclass
-   * @param <S> the Java read support type
+   * @param <S>              the Java read support type
    */
   public <S extends ReadSupport<T>> ParquetInputFormat(Class<S> readSupportClass) {
     this.readSupportClass = readSupportClass;
@@ -274,9 +293,8 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
    * {@inheritDoc}
    */
   @Override
-  public RecordReader<Void, T> createRecordReader(
-      InputSplit inputSplit,
-      TaskAttemptContext taskAttemptContext) throws IOException, InterruptedException {
+  public RecordReader<Void, T> createRecordReader(InputSplit inputSplit, TaskAttemptContext taskAttemptContext)
+      throws IOException, InterruptedException {
     Configuration conf = ContextUtil.getConfiguration(taskAttemptContext);
     ReadSupport<T> readSupport = getReadSupport(conf);
     return new ParquetRecordReader<T>(readSupport, getFilter(conf));
@@ -289,31 +307,30 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
    */
   @Deprecated
   @SuppressWarnings("unchecked")
-  ReadSupport<T> getReadSupport(Configuration configuration){
-    return getReadSupportInstance(readSupportClass == null ?
-        (Class<? extends ReadSupport<T>>) getReadSupportClass(configuration) :
-        readSupportClass);
+  ReadSupport<T> getReadSupport(Configuration configuration) {
+    return getReadSupportInstance(
+        readSupportClass == null
+            ? (Class<? extends ReadSupport<T>>) getReadSupportClass(configuration)
+            : readSupportClass);
   }
 
   /**
    * @param configuration to find the configuration for the read support
-   * @param <T> the Java type of objects created by the ReadSupport
+   * @param <T>           the Java type of objects created by the ReadSupport
    * @return the configured read support
    */
   @SuppressWarnings("unchecked")
-  public static <T> ReadSupport<T> getReadSupportInstance(Configuration configuration){
-    return getReadSupportInstance(
-        (Class<? extends ReadSupport<T>>) getReadSupportClass(configuration));
+  public static <T> ReadSupport<T> getReadSupportInstance(Configuration configuration) {
+    return getReadSupportInstance((Class<? extends ReadSupport<T>>) getReadSupportClass(configuration));
   }
 
   /**
    * @param readSupportClass to instantiate
-   * @param <T> the Java type of objects created by the ReadSupport
+   * @param <T>              the Java type of objects created by the ReadSupport
    * @return the configured read support
    */
   @SuppressWarnings("unchecked")
-  static <T> ReadSupport<T> getReadSupportInstance(
-      Class<? extends ReadSupport<T>> readSupportClass){
+  static <T> ReadSupport<T> getReadSupportInstance(Class<? extends ReadSupport<T>> readSupportClass) {
     try {
       return readSupportClass.newInstance();
     } catch (InstantiationException | IllegalAccessException e) {
@@ -338,12 +355,10 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
       // Although not required by the API, some clients may depend on always
       // receiving ParquetInputSplit. Translation is required at some point.
       for (InputSplit split : super.getSplits(jobContext)) {
-        Preconditions.checkArgument(split instanceof FileSplit,
-            "Cannot wrap non-FileSplit: %s", split);
+        Preconditions.checkArgument(split instanceof FileSplit, "Cannot wrap non-FileSplit: %s", split);
         splits.add(ParquetInputSplit.from((FileSplit) split));
       }
       return splits;
-
     } else {
       splits.addAll(getSplits(configuration, getFooters(jobContext)));
     }
@@ -353,7 +368,7 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
 
   /**
    * @param configuration the configuration to connect to the file system
-   * @param footers the footers of the files to read
+   * @param footers       the footers of the files to read
    * @return the splits for the footers
    * @throws IOException if there is an error while reading
    * @deprecated split planning using file footers will be removed
@@ -364,16 +379,15 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
     final long maxSplitSize = configuration.getLong("mapred.max.split.size", Long.MAX_VALUE);
     final long minSplitSize = Math.max(getFormatMinSplitSize(), configuration.getLong("mapred.min.split.size", 0L));
     if (maxSplitSize < 0 || minSplitSize < 0) {
-      throw new ParquetDecodingException("maxSplitSize or minSplitSize should not be negative: maxSplitSize = " + maxSplitSize + "; minSplitSize = " + minSplitSize);
+      throw new ParquetDecodingException("maxSplitSize or minSplitSize should not be negative: maxSplitSize = "
+          + maxSplitSize + "; minSplitSize = " + minSplitSize);
     }
     GlobalMetaData globalMetaData = ParquetFileWriter.getGlobalMetaData(footers, strictTypeChecking);
-    ReadContext readContext = getReadSupport(configuration).init(new InitContext(
-        configuration,
-        globalMetaData.getKeyValueMetaData(),
-        globalMetaData.getSchema()));
+    ReadContext readContext = getReadSupport(configuration)
+        .init(new InitContext(configuration, globalMetaData.getKeyValueMetaData(), globalMetaData.getSchema()));
 
-    return new ClientSideMetadataSplitStrategy().getSplits(
-        configuration, footers, maxSplitSize, minSplitSize, readContext);
+    return new ClientSideMetadataSplitStrategy()
+        .getSplits(configuration, footers, maxSplitSize, minSplitSize, readContext);
   }
 
   /*
@@ -382,12 +396,11 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
    */
   @Override
   protected List<FileStatus> listStatus(JobContext jobContext) throws IOException {
-    return getAllFileRecursively(super.listStatus(jobContext),
-       ContextUtil.getConfiguration(jobContext));
+    return getAllFileRecursively(super.listStatus(jobContext), ContextUtil.getConfiguration(jobContext));
   }
 
-  private static List<FileStatus> getAllFileRecursively(
-      List<FileStatus> files, Configuration conf) throws IOException {
+  private static List<FileStatus> getAllFileRecursively(List<FileStatus> files, Configuration conf)
+      throws IOException {
     List<FileStatus> result = new ArrayList<FileStatus>();
     for (FileStatus file : files) {
       if (file.isDir()) {
@@ -402,10 +415,9 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
     return result;
   }
 
-  private static void staticAddInputPathRecursively(List<FileStatus> result,
-      FileSystem fs, Path path, PathFilter inputFilter)
-          throws IOException {
-    for (FileStatus stat: fs.listStatus(path, inputFilter)) {
+  private static void staticAddInputPathRecursively(
+      List<FileStatus> result, FileSystem fs, Path path, PathFilter inputFilter) throws IOException {
+    for (FileStatus stat : fs.listStatus(path, inputFilter)) {
       if (stat.isDir()) {
         staticAddInputPathRecursively(result, fs, stat.getPath(), inputFilter);
       } else {
@@ -429,20 +441,18 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
     // footers in the same order as the list of file statuses returned from listStatus()
     Map<FileStatusWrapper, Footer> footersMap = new LinkedHashMap<FileStatusWrapper, Footer>();
     Set<FileStatus> missingStatuses = new HashSet<FileStatus>();
-    Map<Path, FileStatusWrapper> missingStatusesMap =
-            new HashMap<Path, FileStatusWrapper>(missingStatuses.size());
+    Map<Path, FileStatusWrapper> missingStatusesMap = new HashMap<Path, FileStatusWrapper>(missingStatuses.size());
 
     if (footersCache == null) {
-      footersCache =
-              new LruCache<FileStatusWrapper, FootersCacheValue>(Math.max(statuses.size(), MIN_FOOTER_CACHE_SIZE));
+      footersCache = new LruCache<FileStatusWrapper, FootersCacheValue>(
+          Math.max(statuses.size(), MIN_FOOTER_CACHE_SIZE));
     }
     for (FileStatus status : statuses) {
       FileStatusWrapper statusWrapper = new FileStatusWrapper(status);
-      FootersCacheValue cacheEntry =
-              footersCache.getCurrentValue(statusWrapper);
+      FootersCacheValue cacheEntry = footersCache.getCurrentValue(statusWrapper);
       if (LOG.isDebugEnabled()) {
-        LOG.debug("Cache entry " + (cacheEntry == null ? "not " : "")
-                + " found for '" + status.getPath() + "'");
+        LOG.debug(
+            "Cache entry " + (cacheEntry == null ? "not " : "") + " found for '" + status.getPath() + "'");
       }
       if (cacheEntry != null) {
         footersMap.put(statusWrapper, cacheEntry.getFooter());
@@ -452,8 +462,10 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
         missingStatusesMap.put(status.getPath(), statusWrapper);
       }
     }
-    LOG.debug("found {} footers in cache and adding up to {} missing footers to the cache",
-            footersMap.size() ,missingStatuses.size());
+    LOG.debug(
+        "found {} footers in cache and adding up to {} missing footers to the cache",
+        footersMap.size(),
+        missingStatuses.size());
 
     if (!missingStatuses.isEmpty()) {
       List<Footer> newFooters = getFooters(config, missingStatuses);
@@ -472,10 +484,10 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
       Footer footer = footerEntry.getValue();
 
       if (footer == null) {
-          // Footer was originally missing, so get it from the cache again
-          footers.add(footersCache.getCurrentValue(footerEntry.getKey()).getFooter());
+        // Footer was originally missing, so get it from the cache again
+        footers.add(footersCache.getCurrentValue(footerEntry.getKey()).getFooter());
       } else {
-          footers.add(footer);
+        footers.add(footer);
       }
     }
 
@@ -483,13 +495,14 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
   }
 
   public List<Footer> getFooters(Configuration configuration, List<FileStatus> statuses) throws IOException {
-    return getFooters(configuration, (Collection<FileStatus>)statuses);
+    return getFooters(configuration, (Collection<FileStatus>) statuses);
   }
 
   /**
    * the footers for the files
+   *
    * @param configuration to connect to the file system
-   * @param statuses the files to open
+   * @param statuses      the files to open
    * @return the footers of the files
    * @throws IOException if there is an error while reading
    */
@@ -513,8 +526,7 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
    * modification time associated with that footer.  The modification time is
    * used to determine whether the footer is still current.
    */
-  static final class FootersCacheValue
-          implements LruCache.Value<FileStatusWrapper, FootersCacheValue> {
+  static final class FootersCacheValue implements LruCache.Value<FileStatusWrapper, FootersCacheValue> {
     private final long modificationTime;
     private final Footer footer;
 
@@ -528,8 +540,11 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
       long currentModTime = key.getModificationTime();
       boolean isCurrent = modificationTime >= currentModTime;
       if (LOG.isDebugEnabled() && !isCurrent) {
-        LOG.debug("The cache value for '{}' is not current: cached modification time={}, current modification time: {}",
-                key, modificationTime, currentModTime);
+        LOG.debug(
+            "The cache value for '{}' is not current: cached modification time={}, current modification time: {}",
+            key,
+            modificationTime,
+            currentModTime);
       }
       return isCurrent;
     }
@@ -540,8 +555,7 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
 
     @Override
     public boolean isNewerThan(FootersCacheValue otherValue) {
-      return otherValue == null ||
-              modificationTime > otherValue.modificationTime;
+      return otherValue == null || modificationTime > otherValue.modificationTime;
     }
 
     public Path getPath() {
@@ -555,6 +569,7 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
    */
   static final class FileStatusWrapper {
     private final FileStatus status;
+
     public FileStatusWrapper(FileStatus fileStatus) {
       if (fileStatus == null) {
         throw new IllegalArgumentException("FileStatus object cannot be null");
@@ -573,8 +588,7 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
 
     @Override
     public boolean equals(Object other) {
-      return other instanceof FileStatusWrapper &&
-              status.equals(((FileStatusWrapper) other).status);
+      return other instanceof FileStatusWrapper && status.equals(((FileStatusWrapper) other).status);
     }
 
     @Override
@@ -582,15 +596,16 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
       return status.getPath().toString();
     }
   }
-
 }
 
 class ClientSideMetadataSplitStrategy {
-  //Wrapper of hdfs blocks, keep track of which HDFS block is being used
+  // Wrapper of hdfs blocks, keep track of which HDFS block is being used
   private static class HDFSBlocks {
     BlockLocation[] hdfsBlocks;
-    int currentStartHdfsBlockIndex = 0;//the hdfs block index corresponding to the start of a row group
-    int currentMidPointHDFSBlockIndex = 0;// the hdfs block index corresponding to the mid-point of a row group, a split might be created only when the midpoint of the rowgroup enters a new hdfs block
+    int currentStartHdfsBlockIndex = 0; // the hdfs block index corresponding to the start of a row group
+    int currentMidPointHDFSBlockIndex =
+        0; // the hdfs block index corresponding to the mid-point of a row group, a split might be created only
+    // when the midpoint of the rowgroup enters a new hdfs block
 
     private HDFSBlocks(BlockLocation[] hdfsBlocks) {
       this.hdfsBlocks = hdfsBlocks;
@@ -617,12 +632,13 @@ class ClientSideMetadataSplitStrategy {
       boolean isNewHdfsBlock = false;
       long rowGroupMidPoint = rowGroupMetadata.getStartingPos() + (rowGroupMetadata.getCompressedSize() / 2);
 
-      //if mid point is not in the current HDFS block any more, return true
+      // if mid point is not in the current HDFS block any more, return true
       while (rowGroupMidPoint > getHDFSBlockEndingPosition(currentMidPointHDFSBlockIndex)) {
         isNewHdfsBlock = true;
         currentMidPointHDFSBlockIndex++;
         if (currentMidPointHDFSBlockIndex >= hdfsBlocks.length)
-          throw new ParquetDecodingException("the row group is not in hdfs blocks in the file: midpoint of row groups is "
+          throw new ParquetDecodingException(
+              "the row group is not in hdfs blocks in the file: midpoint of row groups is "
                   + rowGroupMidPoint
                   + ", the end of the hdfs block is "
                   + getHDFSBlockEndingPosition(currentMidPointHDFSBlockIndex - 1));
@@ -632,9 +648,9 @@ class ClientSideMetadataSplitStrategy {
         currentStartHdfsBlockIndex++;
         if (currentStartHdfsBlockIndex >= hdfsBlocks.length)
           throw new ParquetDecodingException("The row group does not start in this file: row group offset is "
-                  + rowGroupMetadata.getStartingPos()
-                  + " but the end of hdfs blocks of file is "
-                  + getHDFSBlockEndingPosition(currentStartHdfsBlockIndex));
+              + rowGroupMetadata.getStartingPos()
+              + " but the end of hdfs blocks of file is "
+              + getHDFSBlockEndingPosition(currentStartHdfsBlockIndex));
       }
       return isNewHdfsBlock;
     }
@@ -670,7 +686,9 @@ class ClientSideMetadataSplitStrategy {
       return rowGroups.size();
     }
 
-    public ParquetInputSplit getParquetInputSplit(FileStatus fileStatus, String requestedSchema, Map<String, String> readSupportMetadata) throws IOException {
+    public ParquetInputSplit getParquetInputSplit(
+        FileStatus fileStatus, String requestedSchema, Map<String, String> readSupportMetadata)
+        throws IOException {
       MessageType requested = MessageTypeParser.parseMessageType(requestedSchema);
       long length = 0;
 
@@ -692,20 +710,18 @@ class ClientSideMetadataSplitStrategy {
       }
 
       return new ParquetInputSplit(
-              fileStatus.getPath(),
-              hdfsBlock.getOffset(),
-              end,
-              length,
-              hdfsBlock.getHosts(),
-              rowGroupOffsets
-      );
+          fileStatus.getPath(), hdfsBlock.getOffset(), end, length, hdfsBlock.getHosts(), rowGroupOffsets);
     }
   }
 
   private static final Logger LOG = LoggerFactory.getLogger(ClientSideMetadataSplitStrategy.class);
 
-  List<ParquetInputSplit> getSplits(Configuration configuration, List<Footer> footers,
-      long maxSplitSize, long minSplitSize, ReadContext readContext)
+  List<ParquetInputSplit> getSplits(
+      Configuration configuration,
+      List<Footer> footers,
+      long maxSplitSize,
+      long minSplitSize,
+      ReadContext readContext)
       throws IOException {
     List<ParquetInputSplit> splits = new ArrayList<ParquetInputSplit>();
     Filter filter = ParquetInputFormat.getFilter(configuration);
@@ -724,7 +740,8 @@ class ClientSideMetadataSplitStrategy {
       List<BlockMetaData> filteredBlocks;
 
       totalRowGroups += blocks.size();
-      filteredBlocks = RowGroupFilter.filterRowGroups(filter, blocks, parquetMetaData.getFileMetaData().getSchema());
+      filteredBlocks = RowGroupFilter.filterRowGroups(
+          filter, blocks, parquetMetaData.getFileMetaData().getSchema());
       rowGroupsDropped += blocks.size() - filteredBlocks.size();
 
       if (filteredBlocks.isEmpty()) {
@@ -732,21 +749,21 @@ class ClientSideMetadataSplitStrategy {
       }
 
       BlockLocation[] fileBlockLocations = fs.getFileBlockLocations(fileStatus, 0, fileStatus.getLen());
-      splits.addAll(
-          generateSplits(
-              filteredBlocks,
-              fileBlockLocations,
-              fileStatus,
-              readContext.getRequestedSchema().toString(),
-              readContext.getReadSupportMetadata(),
-              minSplitSize,
-              maxSplitSize)
-          );
+      splits.addAll(generateSplits(
+          filteredBlocks,
+          fileBlockLocations,
+          fileStatus,
+          readContext.getRequestedSchema().toString(),
+          readContext.getReadSupportMetadata(),
+          minSplitSize,
+          maxSplitSize));
     }
 
     if (rowGroupsDropped > 0 && totalRowGroups > 0) {
       int percentDropped = (int) ((((double) rowGroupsDropped) / totalRowGroups) * 100);
-      LOG.info("Dropping {} row groups that do not pass filter predicate! ({}%)", rowGroupsDropped, percentDropped);
+      LOG.info(
+          "Dropping {} row groups that do not pass filter predicate! ({}%)",
+          rowGroupsDropped, percentDropped);
     } else {
       LOG.info("There were no row groups that could be dropped due to filter predicates");
     }
@@ -767,16 +784,18 @@ class ClientSideMetadataSplitStrategy {
    * @throws IOException If hosts can't be retrieved for the HDFS block
    */
   static <T> List<ParquetInputSplit> generateSplits(
-          List<BlockMetaData> rowGroupBlocks,
-          BlockLocation[] hdfsBlocksArray,
-          FileStatus fileStatus,
-          String requestedSchema,
-          Map<String, String> readSupportMetadata, long minSplitSize, long maxSplitSize) throws IOException {
+      List<BlockMetaData> rowGroupBlocks,
+      BlockLocation[] hdfsBlocksArray,
+      FileStatus fileStatus,
+      String requestedSchema,
+      Map<String, String> readSupportMetadata,
+      long minSplitSize,
+      long maxSplitSize)
+      throws IOException {
 
-    List<SplitInfo> splitRowGroups =
-        generateSplitInfo(rowGroupBlocks, hdfsBlocksArray, minSplitSize, maxSplitSize);
+    List<SplitInfo> splitRowGroups = generateSplitInfo(rowGroupBlocks, hdfsBlocksArray, minSplitSize, maxSplitSize);
 
-    //generate splits from rowGroups of each split
+    // generate splits from rowGroups of each split
     List<ParquetInputSplit> resultSplits = new ArrayList<ParquetInputSplit>();
     for (SplitInfo splitInfo : splitRowGroups) {
       ParquetInputSplit split = splitInfo.getParquetInputSplit(fileStatus, requestedSchema, readSupportMetadata);
@@ -786,28 +805,28 @@ class ClientSideMetadataSplitStrategy {
   }
 
   static List<SplitInfo> generateSplitInfo(
-      List<BlockMetaData> rowGroupBlocks,
-      BlockLocation[] hdfsBlocksArray,
-      long minSplitSize, long maxSplitSize) {
+      List<BlockMetaData> rowGroupBlocks, BlockLocation[] hdfsBlocksArray, long minSplitSize, long maxSplitSize) {
     List<SplitInfo> splitRowGroups;
 
     if (maxSplitSize < minSplitSize || maxSplitSize < 0 || minSplitSize < 0) {
-      throw new ParquetDecodingException("maxSplitSize and minSplitSize should be positive and max should be greater or equal to the minSplitSize: maxSplitSize = " + maxSplitSize + "; minSplitSize is " + minSplitSize);
+      throw new ParquetDecodingException(
+          "maxSplitSize and minSplitSize should be positive and max should be greater or equal to the minSplitSize: maxSplitSize = "
+              + maxSplitSize + "; minSplitSize is " + minSplitSize);
     }
     HDFSBlocks hdfsBlocks = new HDFSBlocks(hdfsBlocksArray);
     hdfsBlocks.checkBelongingToANewHDFSBlock(rowGroupBlocks.get(0));
     SplitInfo currentSplit = new SplitInfo(hdfsBlocks.getCurrentBlock());
 
-    //assign rowGroups to splits
+    // assign rowGroups to splits
     splitRowGroups = new ArrayList<SplitInfo>();
-    checkSorted(rowGroupBlocks);//assert row groups are sorted
+    checkSorted(rowGroupBlocks); // assert row groups are sorted
     for (BlockMetaData rowGroupMetadata : rowGroupBlocks) {
       if ((hdfsBlocks.checkBelongingToANewHDFSBlock(rowGroupMetadata)
-             && currentSplit.getCompressedByteSize() >= minSplitSize
-             && currentSplit.getCompressedByteSize() > 0)
-           || currentSplit.getCompressedByteSize() >= maxSplitSize) {
-        //create a new split
-        splitRowGroups.add(currentSplit);//finish previous split
+              && currentSplit.getCompressedByteSize() >= minSplitSize
+              && currentSplit.getCompressedByteSize() > 0)
+          || currentSplit.getCompressedByteSize() >= maxSplitSize) {
+        // create a new split
+        splitRowGroups.add(currentSplit); // finish previous split
         currentSplit = new SplitInfo(hdfsBlocks.getCurrentBlock());
       }
       currentSplit.addRowGroup(rowGroupMetadata);
@@ -822,10 +841,11 @@ class ClientSideMetadataSplitStrategy {
 
   private static void checkSorted(List<BlockMetaData> rowGroupBlocks) {
     long previousOffset = 0L;
-    for(BlockMetaData rowGroup: rowGroupBlocks) {
+    for (BlockMetaData rowGroup : rowGroupBlocks) {
       long currentOffset = rowGroup.getStartingPos();
       if (currentOffset < previousOffset) {
-        throw new ParquetDecodingException("row groups are not sorted: previous row groups starts at " + previousOffset + ", current row group starts at " + currentOffset);
+        throw new ParquetDecodingException("row groups are not sorted: previous row groups starts at "
+            + previousOffset + ", current row group starts at " + currentOffset);
       }
     }
   }
