@@ -20,16 +20,21 @@ package org.apache.parquet.hadoop;
 
 import java.io.Closeable;
 import java.io.IOException;
-
+import java.util.HashMap;
+import java.util.Map;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.Path;
-
+import org.apache.parquet.bytes.ByteBufferAllocator;
 import org.apache.parquet.column.ParquetProperties;
 import org.apache.parquet.column.ParquetProperties.WriterVersion;
+import org.apache.parquet.compression.CompressionCodecFactory;
+import org.apache.parquet.conf.HadoopParquetConfiguration;
+import org.apache.parquet.conf.ParquetConfiguration;
 import org.apache.parquet.crypto.FileEncryptionProperties;
 import org.apache.parquet.hadoop.api.WriteSupport;
 import org.apache.parquet.hadoop.metadata.CompressionCodecName;
 import org.apache.parquet.hadoop.metadata.ParquetMetadata;
+import org.apache.parquet.hadoop.util.ConfigurationUtil;
 import org.apache.parquet.hadoop.util.HadoopOutputFile;
 import org.apache.parquet.io.OutputFile;
 import org.apache.parquet.schema.MessageType;
@@ -40,15 +45,11 @@ import org.apache.parquet.schema.MessageType;
 public class ParquetWriter<T> implements Closeable {
 
   public static final int DEFAULT_BLOCK_SIZE = 128 * 1024 * 1024;
-  public static final int DEFAULT_PAGE_SIZE =
-      ParquetProperties.DEFAULT_PAGE_SIZE;
-  public static final CompressionCodecName DEFAULT_COMPRESSION_CODEC_NAME =
-      CompressionCodecName.UNCOMPRESSED;
-  public static final boolean DEFAULT_IS_DICTIONARY_ENABLED =
-      ParquetProperties.DEFAULT_IS_DICTIONARY_ENABLED;
+  public static final int DEFAULT_PAGE_SIZE = ParquetProperties.DEFAULT_PAGE_SIZE;
+  public static final CompressionCodecName DEFAULT_COMPRESSION_CODEC_NAME = CompressionCodecName.UNCOMPRESSED;
+  public static final boolean DEFAULT_IS_DICTIONARY_ENABLED = ParquetProperties.DEFAULT_IS_DICTIONARY_ENABLED;
   public static final boolean DEFAULT_IS_VALIDATING_ENABLED = false;
-  public static final WriterVersion DEFAULT_WRITER_VERSION =
-      ParquetProperties.DEFAULT_WRITER_VERSION;
+  public static final WriterVersion DEFAULT_WRITER_VERSION = ParquetProperties.DEFAULT_WRITER_VERSION;
 
   public static final String OBJECT_MODEL_NAME_PROP = "writer.model.name";
 
@@ -56,36 +57,48 @@ public class ParquetWriter<T> implements Closeable {
   public static final int MAX_PADDING_SIZE_DEFAULT = 8 * 1024 * 1024; // 8MB
 
   private final InternalParquetRecordWriter<T> writer;
-  private final CodecFactory codecFactory;
+  private final CompressionCodecFactory codecFactory;
 
   /**
    * Create a new ParquetWriter.
    * (with dictionary encoding enabled and validation off)
    *
-   * @param file the file to create
-   * @param writeSupport the implementation to write a record to a RecordConsumer
+   * @param file                 the file to create
+   * @param writeSupport         the implementation to write a record to a RecordConsumer
    * @param compressionCodecName the compression codec to use
-   * @param blockSize the block size threshold
-   * @param pageSize the page size threshold
+   * @param blockSize            the block size threshold
+   * @param pageSize             the page size threshold
    * @throws IOException if there is an error while writing
    * @deprecated will be removed in 2.0.0
    */
   @Deprecated
-  public ParquetWriter(Path file, WriteSupport<T> writeSupport, CompressionCodecName compressionCodecName, int blockSize, int pageSize) throws IOException {
-    this(file, writeSupport, compressionCodecName, blockSize, pageSize,
-        DEFAULT_IS_DICTIONARY_ENABLED, DEFAULT_IS_VALIDATING_ENABLED);
+  public ParquetWriter(
+      Path file,
+      WriteSupport<T> writeSupport,
+      CompressionCodecName compressionCodecName,
+      int blockSize,
+      int pageSize)
+      throws IOException {
+    this(
+        file,
+        writeSupport,
+        compressionCodecName,
+        blockSize,
+        pageSize,
+        DEFAULT_IS_DICTIONARY_ENABLED,
+        DEFAULT_IS_VALIDATING_ENABLED);
   }
 
   /**
    * Create a new ParquetWriter.
    *
-   * @param file the file to create
-   * @param writeSupport the implementation to write a record to a RecordConsumer
+   * @param file                 the file to create
+   * @param writeSupport         the implementation to write a record to a RecordConsumer
    * @param compressionCodecName the compression codec to use
-   * @param blockSize the block size threshold
-   * @param pageSize the page size threshold (both data and dictionary)
-   * @param enableDictionary to turn dictionary encoding on
-   * @param validating to turn on validation using the schema
+   * @param blockSize            the block size threshold
+   * @param pageSize             the page size threshold (both data and dictionary)
+   * @param enableDictionary     to turn dictionary encoding on
+   * @param validating           to turn on validation using the schema
    * @throws IOException if there is an error while writing
    * @deprecated will be removed in 2.0.0
    */
@@ -97,21 +110,22 @@ public class ParquetWriter<T> implements Closeable {
       int blockSize,
       int pageSize,
       boolean enableDictionary,
-      boolean validating) throws IOException {
+      boolean validating)
+      throws IOException {
     this(file, writeSupport, compressionCodecName, blockSize, pageSize, pageSize, enableDictionary, validating);
   }
 
   /**
    * Create a new ParquetWriter.
    *
-   * @param file the file to create
-   * @param writeSupport the implementation to write a record to a RecordConsumer
+   * @param file                 the file to create
+   * @param writeSupport         the implementation to write a record to a RecordConsumer
    * @param compressionCodecName the compression codec to use
-   * @param blockSize the block size threshold
-   * @param pageSize the page size threshold
-   * @param dictionaryPageSize the page size threshold for the dictionary pages
-   * @param enableDictionary to turn dictionary encoding on
-   * @param validating to turn on validation using the schema
+   * @param blockSize            the block size threshold
+   * @param pageSize             the page size threshold
+   * @param dictionaryPageSize   the page size threshold for the dictionary pages
+   * @param enableDictionary     to turn dictionary encoding on
+   * @param validating           to turn on validation using the schema
    * @throws IOException if there is an error while writing
    * @deprecated will be removed in 2.0.0
    */
@@ -124,27 +138,35 @@ public class ParquetWriter<T> implements Closeable {
       int pageSize,
       int dictionaryPageSize,
       boolean enableDictionary,
-      boolean validating) throws IOException {
-    this(file, writeSupport, compressionCodecName, blockSize, pageSize,
-        dictionaryPageSize, enableDictionary, validating,
+      boolean validating)
+      throws IOException {
+    this(
+        file,
+        writeSupport,
+        compressionCodecName,
+        blockSize,
+        pageSize,
+        dictionaryPageSize,
+        enableDictionary,
+        validating,
         DEFAULT_WRITER_VERSION);
   }
 
   /**
    * Create a new ParquetWriter.
-   *
+   * <p>
    * Directly instantiates a Hadoop {@link org.apache.hadoop.conf.Configuration} which reads
    * configuration from the classpath.
    *
-   * @param file the file to create
-   * @param writeSupport the implementation to write a record to a RecordConsumer
+   * @param file                 the file to create
+   * @param writeSupport         the implementation to write a record to a RecordConsumer
    * @param compressionCodecName the compression codec to use
-   * @param blockSize the block size threshold
-   * @param pageSize the page size threshold
-   * @param dictionaryPageSize the page size threshold for the dictionary pages
-   * @param enableDictionary to turn dictionary encoding on
-   * @param validating to turn on validation using the schema
-   * @param writerVersion version of parquetWriter from {@link ParquetProperties.WriterVersion}
+   * @param blockSize            the block size threshold
+   * @param pageSize             the page size threshold
+   * @param dictionaryPageSize   the page size threshold for the dictionary pages
+   * @param enableDictionary     to turn dictionary encoding on
+   * @param validating           to turn on validation using the schema
+   * @param writerVersion        version of parquetWriter from {@link ParquetProperties.WriterVersion}
    * @throws IOException if there is an error while writing
    * @deprecated will be removed in 2.0.0
    */
@@ -158,23 +180,34 @@ public class ParquetWriter<T> implements Closeable {
       int dictionaryPageSize,
       boolean enableDictionary,
       boolean validating,
-      WriterVersion writerVersion) throws IOException {
-    this(file, writeSupport, compressionCodecName, blockSize, pageSize, dictionaryPageSize, enableDictionary, validating, writerVersion, new Configuration());
+      WriterVersion writerVersion)
+      throws IOException {
+    this(
+        file,
+        writeSupport,
+        compressionCodecName,
+        blockSize,
+        pageSize,
+        dictionaryPageSize,
+        enableDictionary,
+        validating,
+        writerVersion,
+        new Configuration());
   }
 
   /**
    * Create a new ParquetWriter.
    *
-   * @param file the file to create
-   * @param writeSupport the implementation to write a record to a RecordConsumer
+   * @param file                 the file to create
+   * @param writeSupport         the implementation to write a record to a RecordConsumer
    * @param compressionCodecName the compression codec to use
-   * @param blockSize the block size threshold
-   * @param pageSize the page size threshold
-   * @param dictionaryPageSize the page size threshold for the dictionary pages
-   * @param enableDictionary to turn dictionary encoding on
-   * @param validating to turn on validation using the schema
-   * @param writerVersion version of parquetWriter from {@link ParquetProperties.WriterVersion}
-   * @param conf Hadoop configuration to use while accessing the filesystem
+   * @param blockSize            the block size threshold
+   * @param pageSize             the page size threshold
+   * @param dictionaryPageSize   the page size threshold for the dictionary pages
+   * @param enableDictionary     to turn dictionary encoding on
+   * @param validating           to turn on validation using the schema
+   * @param writerVersion        version of parquetWriter from {@link ParquetProperties.WriterVersion}
+   * @param conf                 Hadoop configuration to use while accessing the filesystem
    * @throws IOException if there is an error while writing
    * @deprecated will be removed in 2.0.0
    */
@@ -189,26 +222,36 @@ public class ParquetWriter<T> implements Closeable {
       boolean enableDictionary,
       boolean validating,
       WriterVersion writerVersion,
-      Configuration conf) throws IOException {
-    this(file, ParquetFileWriter.Mode.CREATE, writeSupport,
-        compressionCodecName, blockSize, pageSize, dictionaryPageSize,
-        enableDictionary, validating, writerVersion, conf);
+      Configuration conf)
+      throws IOException {
+    this(
+        file,
+        ParquetFileWriter.Mode.CREATE,
+        writeSupport,
+        compressionCodecName,
+        blockSize,
+        pageSize,
+        dictionaryPageSize,
+        enableDictionary,
+        validating,
+        writerVersion,
+        conf);
   }
 
   /**
    * Create a new ParquetWriter.
    *
-   * @param file the file to create
-   * @param mode file creation mode
-   * @param writeSupport the implementation to write a record to a RecordConsumer
+   * @param file                 the file to create
+   * @param mode                 file creation mode
+   * @param writeSupport         the implementation to write a record to a RecordConsumer
    * @param compressionCodecName the compression codec to use
-   * @param blockSize the block size threshold
-   * @param pageSize the page size threshold
-   * @param dictionaryPageSize the page size threshold for the dictionary pages
-   * @param enableDictionary to turn dictionary encoding on
-   * @param validating to turn on validation using the schema
-   * @param writerVersion version of parquetWriter from {@link ParquetProperties.WriterVersion}
-   * @param conf Hadoop configuration to use while accessing the filesystem
+   * @param blockSize            the block size threshold
+   * @param pageSize             the page size threshold
+   * @param dictionaryPageSize   the page size threshold for the dictionary pages
+   * @param enableDictionary     to turn dictionary encoding on
+   * @param validating           to turn on validation using the schema
+   * @param writerVersion        version of parquetWriter from {@link ParquetProperties.WriterVersion}
+   * @param conf                 Hadoop configuration to use while accessing the filesystem
    * @throws IOException if there is an error while writing
    * @deprecated will be removed in 2.0.0
    */
@@ -224,23 +267,31 @@ public class ParquetWriter<T> implements Closeable {
       boolean enableDictionary,
       boolean validating,
       WriterVersion writerVersion,
-      Configuration conf) throws IOException {
-    this(HadoopOutputFile.fromPath(file, conf),
-        mode, writeSupport, compressionCodecName, blockSize,
-        validating, conf, MAX_PADDING_SIZE_DEFAULT,
+      Configuration conf)
+      throws IOException {
+    this(
+        HadoopOutputFile.fromPath(file, conf),
+        mode,
+        writeSupport,
+        compressionCodecName,
+        blockSize,
+        validating,
+        conf,
+        MAX_PADDING_SIZE_DEFAULT,
         ParquetProperties.builder()
             .withPageSize(pageSize)
             .withDictionaryPageSize(dictionaryPageSize)
             .withDictionaryEncoding(enableDictionary)
             .withWriterVersion(writerVersion)
-            .build(), null);
+            .build(),
+        null);
   }
 
   /**
-   * Create a new ParquetWriter.  The default block size is 50 MB.The default
-   * page size is 1 MB.  Default compression is no compression. Dictionary encoding is disabled.
+   * Create a new ParquetWriter. The default block size is 128 MB. The default
+   * page size is 1 MB. Default compression is no compression. Dictionary encoding is disabled.
    *
-   * @param file the file to create
+   * @param file         the file to create
    * @param writeSupport the implementation to write a record to a RecordConsumer
    * @throws IOException if there is an error while writing
    * @deprecated will be removed in 2.0.0
@@ -252,7 +303,8 @@ public class ParquetWriter<T> implements Closeable {
 
   @Deprecated
   public ParquetWriter(Path file, Configuration conf, WriteSupport<T> writeSupport) throws IOException {
-    this(file,
+    this(
+        file,
         writeSupport,
         DEFAULT_COMPRESSION_CODEC_NAME,
         DEFAULT_BLOCK_SIZE,
@@ -274,35 +326,101 @@ public class ParquetWriter<T> implements Closeable {
       Configuration conf,
       int maxPaddingSize,
       ParquetProperties encodingProps,
-      FileEncryptionProperties encryptionProperties) throws IOException {
+      FileEncryptionProperties encryptionProperties)
+      throws IOException {
+    this(
+        file,
+        mode,
+        writeSupport,
+        compressionCodecName,
+        rowGroupSize,
+        validating,
+        new HadoopParquetConfiguration(conf),
+        maxPaddingSize,
+        encodingProps,
+        encryptionProperties);
+  }
 
+  ParquetWriter(
+      OutputFile file,
+      ParquetFileWriter.Mode mode,
+      WriteSupport<T> writeSupport,
+      CompressionCodecName compressionCodecName,
+      long rowGroupSize,
+      boolean validating,
+      ParquetConfiguration conf,
+      int maxPaddingSize,
+      ParquetProperties encodingProps,
+      FileEncryptionProperties encryptionProperties)
+      throws IOException {
+    this(
+        file,
+        mode,
+        writeSupport,
+        compressionCodecName,
+        new CodecFactory(conf, encodingProps.getPageSizeThreshold()),
+        rowGroupSize,
+        validating,
+        conf,
+        maxPaddingSize,
+        encodingProps,
+        encryptionProperties);
+  }
+
+  ParquetWriter(
+      OutputFile file,
+      ParquetFileWriter.Mode mode,
+      WriteSupport<T> writeSupport,
+      CompressionCodecName compressionCodecName,
+      CompressionCodecFactory codecFactory,
+      long rowGroupSize,
+      boolean validating,
+      ParquetConfiguration conf,
+      int maxPaddingSize,
+      ParquetProperties encodingProps,
+      FileEncryptionProperties encryptionProperties)
+      throws IOException {
     WriteSupport.WriteContext writeContext = writeSupport.init(conf);
     MessageType schema = writeContext.getSchema();
 
-    // encryptionProperties could be built from the implementation of EncryptionPropertiesFactory when it is attached.
+    // encryptionProperties could be built from the implementation of EncryptionPropertiesFactory when it is
+    // attached.
     if (encryptionProperties == null) {
       String path = file == null ? null : file.getPath();
-      encryptionProperties = ParquetOutputFormat.createEncryptionProperties(conf,
-          path == null ? null : new Path(path), writeContext);
+      Configuration hadoopConf = ConfigurationUtil.createHadoopConfiguration(conf);
+      encryptionProperties = ParquetOutputFormat.createEncryptionProperties(
+          hadoopConf, path == null ? null : new Path(path), writeContext);
     }
 
     ParquetFileWriter fileWriter = new ParquetFileWriter(
-      file, schema, mode, rowGroupSize, maxPaddingSize,
-      encodingProps.getColumnIndexTruncateLength(), encodingProps.getStatisticsTruncateLength(),
-      encodingProps.getPageWriteChecksumEnabled(), encryptionProperties);
+        file, schema, mode, rowGroupSize, maxPaddingSize, encryptionProperties, encodingProps);
     fileWriter.start();
 
-    this.codecFactory = new CodecFactory(conf, encodingProps.getPageSizeThreshold());
-    CodecFactory.BytesCompressor compressor =	codecFactory.getCompressor(compressionCodecName);
+    this.codecFactory = codecFactory;
+    CompressionCodecFactory.BytesInputCompressor compressor = codecFactory.getCompressor(compressionCodecName);
+
+    final Map<String, String> extraMetadata;
+    if (encodingProps.getExtraMetaData() == null
+        || encodingProps.getExtraMetaData().isEmpty()) {
+      extraMetadata = writeContext.getExtraMetaData();
+    } else {
+      extraMetadata = new HashMap<>(writeContext.getExtraMetaData());
+
+      encodingProps.getExtraMetaData().forEach((metadataKey, metadataValue) -> {
+        if (metadataKey.equals(OBJECT_MODEL_NAME_PROP)) {
+          throw new IllegalArgumentException("Cannot overwrite metadata key " + OBJECT_MODEL_NAME_PROP
+              + ". Please use another key name.");
+        }
+
+        if (extraMetadata.put(metadataKey, metadataValue) != null) {
+          throw new IllegalArgumentException(
+              "Duplicate metadata key " + metadataKey + ". Please use another key name.");
+        }
+      });
+    }
+
     this.writer = new InternalParquetRecordWriter<T>(
-        fileWriter,
-        writeSupport,
-        schema,
-        writeContext.getExtraMetaData(),
-        rowGroupSize,
-        compressor,
-        validating,
-        encodingProps);
+        fileWriter, writeSupport, schema, extraMetadata, rowGroupSize, compressor, validating, encodingProps);
   }
 
   public void write(T object) throws IOException {
@@ -341,25 +459,25 @@ public class ParquetWriter<T> implements Closeable {
 
   /**
    * An abstract builder class for ParquetWriter instances.
-   *
+   * <p>
    * Object models should extend this builder to provide writer configuration
    * options.
    *
-   * @param <T> The type of objects written by the constructed ParquetWriter.
+   * @param <T>    The type of objects written by the constructed ParquetWriter.
    * @param <SELF> The type of this builder that is returned by builder methods
    */
   public abstract static class Builder<T, SELF extends Builder<T, SELF>> {
     private OutputFile file = null;
     private Path path = null;
     private FileEncryptionProperties encryptionProperties = null;
-    private Configuration conf = new Configuration();
+    private ParquetConfiguration conf = null;
     private ParquetFileWriter.Mode mode;
+    private CompressionCodecFactory codecFactory = null;
     private CompressionCodecName codecName = DEFAULT_COMPRESSION_CODEC_NAME;
     private long rowGroupSize = DEFAULT_BLOCK_SIZE;
     private int maxPaddingSize = MAX_PADDING_SIZE_DEFAULT;
     private boolean enableValidation = DEFAULT_IS_VALIDATING_ENABLED;
-    private ParquetProperties.Builder encodingPropsBuilder =
-        ParquetProperties.builder();
+    private ParquetProperties.Builder encodingPropsBuilder = ParquetProperties.builder();
 
     protected Builder(Path path) {
       this.path = path;
@@ -377,9 +495,19 @@ public class ParquetWriter<T> implements Closeable {
     /**
      * @param conf a configuration
      * @return an appropriate WriteSupport for the object model.
+     * @deprecated Use {@link #getWriteSupport(ParquetConfiguration)} instead
      */
+    @Deprecated
     protected abstract WriteSupport<T> getWriteSupport(Configuration conf);
 
+    /**
+     * @param conf a configuration
+     * @return an appropriate WriteSupport for the object model.
+     */
+    protected WriteSupport<T> getWriteSupport(ParquetConfiguration conf) {
+      return getWriteSupport(ConfigurationUtil.createHadoopConfiguration(conf));
+    }
+
     /**
      * Set the {@link Configuration} used by the constructed writer.
      *
@@ -387,6 +515,17 @@ public class ParquetWriter<T> implements Closeable {
      * @return this builder for method chaining.
      */
     public SELF withConf(Configuration conf) {
+      this.conf = new HadoopParquetConfiguration(conf);
+      return self();
+    }
+
+    /**
+     * Set the {@link ParquetConfiguration} used by the constructed writer.
+     *
+     * @param conf a {@code ParquetConfiguration}
+     * @return this builder for method chaining.
+     */
+    public SELF withConf(ParquetConfiguration conf) {
       this.conf = conf;
       return self();
     }
@@ -415,6 +554,18 @@ public class ParquetWriter<T> implements Closeable {
       return self();
     }
 
+    /**
+     * Set the {@link CompressionCodecFactory codec factory} used by the
+     * constructed writer.
+     *
+     * @param codecFactory a {@link CompressionCodecFactory}
+     * @return this builder for method chaining.
+     */
+    public SELF withCodecFactory(CompressionCodecFactory codecFactory) {
+      this.codecFactory = codecFactory;
+      return self();
+    }
+
     /**
      * Set the {@link FileEncryptionProperties file encryption properties} used by the
      * constructed writer.
@@ -422,7 +573,7 @@ public class ParquetWriter<T> implements Closeable {
      * @param encryptionProperties a {@code FileEncryptionProperties}
      * @return this builder for method chaining.
      */
-    public SELF withEncryption (FileEncryptionProperties encryptionProperties) {
+    public SELF withEncryption(FileEncryptionProperties encryptionProperties) {
       this.encryptionProperties = encryptionProperties;
       return self();
     }
@@ -526,7 +677,7 @@ public class ParquetWriter<T> implements Closeable {
     /**
      * Enable or disable dictionary encoding of the specified column for the constructed writer.
      *
-     * @param columnPath the path of the column (dot-string)
+     * @param columnPath       the path of the column (dot-string)
      * @param enableDictionary whether dictionary encoding should be enabled
      * @return this builder for method chaining.
      */
@@ -589,12 +740,22 @@ public class ParquetWriter<T> implements Closeable {
       return self();
     }
 
+    /**
+     * Set max Bloom filter bytes for related columns.
+     *
+     * @param maxBloomFilterBytes the max bytes of a Bloom filter bitset for a column.
+     * @return this builder for method chaining
+     */
+    public SELF withMaxBloomFilterBytes(int maxBloomFilterBytes) {
+      encodingPropsBuilder.withMaxBloomFilterBytes(maxBloomFilterBytes);
+      return self();
+    }
+
     /**
      * Sets the NDV (number of distinct values) for the specified column.
      *
      * @param columnPath the path of the column (dot-string)
      * @param ndv        the NDV of the column
-     *
      * @return this builder for method chaining.
      */
     public SELF withBloomFilterNDV(String columnPath, long ndv) {
@@ -603,6 +764,33 @@ public class ParquetWriter<T> implements Closeable {
       return self();
     }
 
+    public SELF withBloomFilterFPP(String columnPath, double fpp) {
+      encodingPropsBuilder.withBloomFilterFPP(columnPath, fpp);
+      return self();
+    }
+
+    /**
+     * When NDV (number of distinct values) for a specified column is not set, whether to use
+     * `AdaptiveBloomFilter` to automatically adjust the BloomFilter size according to `parquet.bloom.filter.max.bytes`
+     *
+     * @param enabled whether to write bloom filter for the column
+     */
+    public SELF withAdaptiveBloomFilterEnabled(boolean enabled) {
+      encodingPropsBuilder.withAdaptiveBloomFilterEnabled(enabled);
+      return self();
+    }
+
+    /**
+     * When `AdaptiveBloomFilter` is enabled, set how many bloom filter candidates to use.
+     *
+     * @param columnPath the path of the column (dot-string)
+     * @param number     the number of candidate
+     */
+    public SELF withBloomFilterCandidateNumber(String columnPath, int number) {
+      encodingPropsBuilder.withBloomFilterCandidatesNumber(columnPath, number);
+      return self();
+    }
+
     /**
      * Sets the bloom filter enabled/disabled
      *
@@ -627,15 +815,84 @@ public class ParquetWriter<T> implements Closeable {
       return self();
     }
 
+    /**
+     * Sets the minimum number of rows to write before a page size check is done.
+     *
+     * @param min writes at least `min` rows before invoking a page size check
+     * @return this builder for method chaining
+     */
+    public SELF withMinRowCountForPageSizeCheck(int min) {
+      encodingPropsBuilder.withMinRowCountForPageSizeCheck(min);
+      return self();
+    }
+
+    /**
+     * Sets the maximum number of rows to write before a page size check is done.
+     *
+     * @param max makes a page size check after `max` rows have been written
+     * @return this builder for method chaining
+     */
+    public SELF withMaxRowCountForPageSizeCheck(int max) {
+      encodingPropsBuilder.withMaxRowCountForPageSizeCheck(max);
+      return self();
+    }
+
+    /**
+     * Sets the length to be used for truncating binary values in a binary column index.
+     *
+     * @param length the length to truncate to
+     * @return this builder for method chaining
+     */
+    public SELF withColumnIndexTruncateLength(int length) {
+      encodingPropsBuilder.withColumnIndexTruncateLength(length);
+      return self();
+    }
+
+    /**
+     * Sets the length which the min/max binary values in row groups are truncated to.
+     *
+     * @param length the length to truncate to
+     * @return this builder for method chaining
+     */
+    public SELF withStatisticsTruncateLength(int length) {
+      encodingPropsBuilder.withStatisticsTruncateLength(length);
+      return self();
+    }
+
+    /**
+     * Sets additional metadata entries to be included in the file footer.
+     *
+     * @param extraMetaData a Map of additional stringly-typed metadata entries
+     * @return this builder for method chaining
+     */
+    public SELF withExtraMetaData(Map<String, String> extraMetaData) {
+      encodingPropsBuilder.withExtraMetaData(extraMetaData);
+      return self();
+    }
+
+    /**
+     * Sets the ByteBuffer allocator instance to be used for allocating memory for writing.
+     *
+     * @param allocator the allocator instance
+     * @return this builder for method chaining
+     */
+    public SELF withAllocator(ByteBufferAllocator allocator) {
+      encodingPropsBuilder.withAllocator(allocator);
+      return self();
+    }
+
     /**
      * Set a property that will be available to the read path. For writers that use a Hadoop
      * configuration, this is the recommended way to add configuration values.
      *
      * @param property a String property name
-     * @param value a String property value
+     * @param value    a String property value
      * @return this builder for method chaining.
      */
     public SELF config(String property, String value) {
+      if (conf == null) {
+        conf = new HadoopParquetConfiguration();
+      }
       conf.set(property, value);
       return self();
     }
@@ -647,15 +904,39 @@ public class ParquetWriter<T> implements Closeable {
      * @throws IOException if there is an error while creating the writer
      */
     public ParquetWriter<T> build() throws IOException {
+      if (conf == null) {
+        conf = new HadoopParquetConfiguration();
+      }
+      ParquetProperties encodingProps = encodingPropsBuilder.build();
+      if (codecFactory == null) {
+        codecFactory = new CodecFactory(conf, encodingProps.getPageSizeThreshold());
+      }
       if (file != null) {
-        return new ParquetWriter<>(file,
-            mode, getWriteSupport(conf), codecName, rowGroupSize, enableValidation, conf,
-            maxPaddingSize, encodingPropsBuilder.build(), encryptionProperties);
+        return new ParquetWriter<>(
+            file,
+            mode,
+            getWriteSupport(conf),
+            codecName,
+            codecFactory,
+            rowGroupSize,
+            enableValidation,
+            conf,
+            maxPaddingSize,
+            encodingProps,
+            encryptionProperties);
       } else {
-        return new ParquetWriter<>(HadoopOutputFile.fromPath(path, conf),
-            mode, getWriteSupport(conf), codecName,
-            rowGroupSize, enableValidation, conf, maxPaddingSize,
-            encodingPropsBuilder.build(), encryptionProperties);
+        return new ParquetWriter<>(
+            HadoopOutputFile.fromPath(path, ConfigurationUtil.createHadoopConfiguration(conf)),
+            mode,
+            getWriteSupport(conf),
+            codecName,
+            codecFactory,
+            rowGroupSize,
+            enableValidation,
+            conf,
+            maxPaddingSize,
+            encodingProps,
+            encryptionProperties);
       }
     }
   }
