@@ -18,18 +18,12 @@
  */
 package org.apache.parquet.hadoop.metadata;
 
+import com.fasterxml.jackson.databind.ObjectMapper;
 import java.io.IOException;
 import java.io.StringReader;
 import java.io.StringWriter;
 import java.util.List;
 
-import com.fasterxml.jackson.core.JsonGenerationException;
-import com.fasterxml.jackson.core.JsonParseException;
-import com.fasterxml.jackson.databind.JsonMappingException;
-import com.fasterxml.jackson.databind.ObjectMapper;
-import com.fasterxml.jackson.databind.SerializationFeature;
-
-
 /**
  * Meta Data block stored in the footer of the file
  * contains file level (Codec, Schema, ...) and block level (location, columns, record count, ...) meta data
@@ -38,12 +32,6 @@ public class ParquetMetadata {
 
   private static final ObjectMapper objectMapper = new ObjectMapper();
 
-  // Enable FAIL_ON_EMPTY_BEANS on objectmapper. Without this feature parquet-casdacing tests fail,
-  // because LogicalTypeAnnotation implementations are classes without any property.
-  static {
-    objectMapper.disable(SerializationFeature.FAIL_ON_EMPTY_BEANS);
-  }
-
   /**
    * @param parquetMetaData an instance of parquet metadata to convert
    * @return the json representation
@@ -53,7 +41,6 @@ public class ParquetMetadata {
   }
 
   /**
-   *
    * @param parquetMetaData an instance of parquet metadata to convert
    * @return the pretty printed json representation
    */
@@ -62,13 +49,12 @@ public class ParquetMetadata {
   }
 
   private static String toJSON(ParquetMetadata parquetMetaData, boolean isPrettyPrint) {
-    StringWriter stringWriter = new StringWriter();
-
-    try {
+    try (StringWriter stringWriter = new StringWriter()) {
       if (isPrettyPrint) {
         Object objectToPrint;
-        if (parquetMetaData.getFileMetaData() == null ||
-            parquetMetaData.getFileMetaData().getEncryptionType() == FileMetaData.EncryptionType.UNENCRYPTED) {
+        if (parquetMetaData.getFileMetaData() == null
+            || parquetMetaData.getFileMetaData().getEncryptionType()
+                == FileMetaData.EncryptionType.UNENCRYPTED) {
           objectToPrint = parquetMetaData;
         } else {
           objectToPrint = parquetMetaData.getFileMetaData();
@@ -77,14 +63,13 @@ public class ParquetMetadata {
       } else {
         objectMapper.writeValue(stringWriter, parquetMetaData);
       }
+      return stringWriter.toString();
     } catch (IOException e) {
       throw new RuntimeException(e);
     }
-    return stringWriter.toString();
   }
 
   /**
-   *
    * @param json the json representation
    * @return the parsed object
    */
@@ -100,9 +85,8 @@ public class ParquetMetadata {
   private final List<BlockMetaData> blocks;
 
   /**
-   *
    * @param fileMetaData file level metadata
-   * @param blocks block level metadata
+   * @param blocks       block level metadata
    */
   public ParquetMetadata(FileMetaData fileMetaData, List<BlockMetaData> blocks) {
     this.fileMetaData = fileMetaData;
@@ -110,7 +94,6 @@ public class ParquetMetadata {
   }
 
   /**
-   *
    * @return block level metadata
    */
   public List<BlockMetaData> getBlocks() {
@@ -118,17 +101,14 @@ public class ParquetMetadata {
   }
 
   /**
-   *
    * @return file level meta data
    */
   public FileMetaData getFileMetaData() {
     return fileMetaData;
   }
 
-
   @Override
   public String toString() {
-    return "ParquetMetaData{"+fileMetaData+", blocks: "+blocks+"}";
+    return "ParquetMetaData{" + fileMetaData + ", blocks: " + blocks + "}";
   }
-
 }
